@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2006 - 2007 Pekka Riikonen
+  Copyright (C) 2006 - 2014 Pekka Riikonen
 
   The contents of this file are subject to one of the Licenses specified 
   in the COPYING file;  You may not use this file except in compliance 
@@ -262,6 +262,9 @@ static void silc_client_rekey_completion(SilcSKE ske,
     silc_ske_free(conn->internal->ske);
     conn->internal->ske = NULL;
     silc_fsm_finish(fsm);
+
+    /* Close connection */
+    silc_client_close_connection(client, conn);
     return;
   }
 
@@ -465,8 +468,7 @@ SILC_FSM_STATE(silc_client_st_connect_key_exchange)
 {
   SilcClientConnection conn = fsm_context;
   SilcClient client = conn->client;
-  SilcSKEParamsStruct params;
-  SilcClientID cid;
+  SilcSKEParamsStruct params = {};
 
   SILC_LOG_DEBUG(("Starting key exchange protocol"));
 
@@ -507,12 +509,6 @@ SILC_FSM_STATE(silc_client_st_connect_key_exchange)
     /** Run key exchange (TCP) */
     silc_fsm_next(fsm, silc_client_st_connect_auth_resolve);
 
-  /* Old server version requires empty Client ID in packets.  Remove this
-     backwards support somepoint after 1.1 server is released. */
-  memset(&cid, 0, sizeof(cid));
-  cid.ip.data_len = 4;
-  silc_packet_set_ids(conn->stream, SILC_ID_CLIENT, &cid, 0, NULL);
-
   SILC_FSM_CALL(conn->internal->op = silc_ske_initiator(conn->internal->ske,
 							conn->stream,
 							&params, NULL));
@@ -789,9 +785,8 @@ SILC_FSM_STATE(silc_client_st_rekey)
 
   /* Allocate SKE */
   conn->internal->ske =
-    silc_ske_alloc(client->rng, conn->internal->schedule,
-		   conn->internal->params.repository,
-		   conn->public_key, conn->private_key, fsm);
+    silc_ske_alloc(client->rng, conn->internal->schedule, NULL,
+		   conn->public_key, NULL, fsm);
   if (!conn->internal->ske)
     return SILC_FSM_FINISH;
 
