@@ -1,25 +1,25 @@
 /*
 
-  silcattrs.c 
+  silcattrs.c
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2002 Pekka Riikonen
+  Copyright (C) 2002 - 2014 Pekka Riikonen
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  The contents of this file are subject to one of the Licenses specified 
+  in the COPYING file;  You may not use this file except in compliance 
+  with the License.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  The software distributed under the License is distributed on an "AS IS"
+  basis, in the hope that it will be useful, but WITHOUT WARRANTY OF ANY
+  KIND, either expressed or implied.  See the COPYING file for more
+  information.
 
 */
 /* Implementation of Attribute Payload routines */
-/* $Id: silcattrs.c,v 1.12 2002/10/23 16:18:14 priikone Exp $ */
+/* $Id$ */
 
-#include "silcincludes.h"
+#include "silc.h"
 #include "silcattrs.h"
 
 /******************************************************************************
@@ -46,11 +46,14 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
 {
   SilcBuffer tmpbuf = NULL;
   unsigned char tmp[4], *str = NULL, *ret;
-  int len;
+  SilcUInt32 len;
+
+  if (ret_len)
+    *ret_len = 0;
 
   /* Encode according to attribute type */
   if (flags & SILC_ATTRIBUTE_FLAG_VALID) {
-    if (!object && !object_size)
+    if (!object || !object_size)
       return NULL;
 
     switch (attribute) {
@@ -70,25 +73,32 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
     case SILC_ATTRIBUTE_SERVICE:
       {
 	SilcAttributeObjService *service = object;
+	SilcUInt32 len2;
 	if (object_size != sizeof(*service))
 	  return NULL;
 	len = strlen(service->address);
-	str = silc_malloc(7 + len);
-	if (!str)
+	len2 = strlen(service->signon);
+	tmpbuf = silc_buffer_alloc_size(13 + len + len2);
+	if (!tmpbuf)
 	  return NULL;
-	SILC_PUT32_MSB(service->port, str);
-	SILC_PUT16_MSB(len, str + 4);
-	memcpy(str + 6, service->address, len);
-	str[6 + len] = service->status;
-	object = str;
-	object_size = 7 + len;
+	silc_buffer_format(tmpbuf,
+			   SILC_STR_UI_INT(service->port),
+			   SILC_STR_UI_SHORT(len),
+			   SILC_STR_UI_XNSTRING(service->address, len),
+			   SILC_STR_UI_CHAR(service->status),
+			   SILC_STR_UI_SHORT(len2),
+			   SILC_STR_UI_XNSTRING(service->signon, len2),
+			   SILC_STR_UI_INT(service->idle),
+			   SILC_STR_END);
+	object = tmpbuf->data;
+	object_size = silc_buffer_len(tmpbuf);
       }
       break;
 
     case SILC_ATTRIBUTE_STATUS_MOOD:
     case SILC_ATTRIBUTE_PREFERRED_CONTACT:
       {
-	SilcUInt32 mask = (SilcUInt32)object;
+	SilcUInt32 mask = SILC_PTR_TO_32(object);
 	if (object_size != sizeof(SilcUInt32))
 	  return NULL;
 	SILC_PUT32_MSB(mask, tmp);
@@ -114,19 +124,22 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
 
     case SILC_ATTRIBUTE_STATUS_MESSAGE:
     case SILC_ATTRIBUTE_EXTENSION:
+    case SILC_ATTRIBUTE_USER_ICON:
       {
-	SilcAttributeObjMime *mime = object;
+	SilcMime mime = object;
 	if (object_size != sizeof(*mime))
 	  return NULL;
-	object = (void *)mime->mime;
-	object_size = mime->mime_len;
+	str = silc_mime_encode(mime, &object_size);
+	if (!str)
+	  return NULL;
+	object = str;
       }
       break;
 
     case SILC_ATTRIBUTE_GEOLOCATION:
       {
 	SilcAttributeObjGeo *geo = object;
-	int len1, len2, len3, len4;
+	SilcUInt32 len1, len2, len3, len4;
 	if (object_size != sizeof(*geo))
 	  return NULL;
 	len1 = (geo->longitude ? strlen(geo->longitude) : 0);
@@ -150,14 +163,14 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
 			   SILC_STR_UI16_STRING(len4 ? geo->accuracy : ""),
 			   SILC_STR_END);
 	object = tmpbuf->data;
-	object_size = tmpbuf->len;
+	object_size = silc_buffer_len(tmpbuf);
       }
       break;
 
     case SILC_ATTRIBUTE_DEVICE_INFO:
       {
 	SilcAttributeObjDevice *dev = object;
-	int len1, len2, len3, len4;
+	SilcUInt32 len1, len2, len3, len4;
 	if (object_size != sizeof(*dev))
 	  return NULL;
 	len1 = (dev->manufacturer ? strlen(dev->manufacturer) : 0);
@@ -182,7 +195,28 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
 			   SILC_STR_UI16_STRING(len4 ? dev->language : ""),
 			   SILC_STR_END);
 	object = tmpbuf->data;
-	object_size = tmpbuf->len;
+	object_size = silc_buffer_len(tmpbuf);
+      }
+      break;
+
+    case SILC_ATTRIBUTE_PHONE_NUMBER:
+      {
+	SilcAttributeObjPN *pn = object;
+	if (object_size != sizeof(*pn))
+	  return NULL;
+	if (!pn->number || strlen(pn->number) < 5)
+	  return NULL;
+	tmpbuf = silc_buffer_alloc(0);
+	if (!tmpbuf)
+	  return NULL;
+	if (silc_buffer_format(tmpbuf,
+			       SILC_STR_UI_INT(pn->format),
+			       SILC_STR_UI_SHORT(strlen(pn->number)),
+			       SILC_STR_UI16_STRING(pn->number),
+			       SILC_STR_END) < 0)
+	  return NULL;
+	object = tmpbuf->data;
+	object_size = silc_buffer_len(tmpbuf);
       }
       break;
 
@@ -202,7 +236,7 @@ silc_attribute_payload_encode_int(SilcAttribute attribute,
 			   SILC_STR_UI_XNSTRING(pk->data, pk->data_len),
 			   SILC_STR_END);
 	object = tmpbuf->data;
-	object_size = tmpbuf->len;
+	object_size = silc_buffer_len(tmpbuf);
       }
       break;
 
@@ -256,11 +290,11 @@ SilcAttributePayload silc_attribute_payload_alloc(SilcAttribute attribute,
   attr->data =
     silc_attribute_payload_encode_int(attribute, flags, object,
 				      object_size, &tmp_len);
-  attr->data_len = (SilcUInt16)tmp_len;
   if (!attr->data) {
     silc_free(attr);
     return NULL;
   }
+  attr->data_len = (SilcUInt16)tmp_len;
 
   return attr;
 }
@@ -273,39 +307,40 @@ SilcDList silc_attribute_payload_parse(const unsigned char *payload,
   SilcBufferStruct buffer;
   SilcDList list;
   SilcAttributePayload newp;
-  int len, ret;
+  SilcUInt32 len;
+  int ret;
 
   SILC_LOG_DEBUG(("Parsing Attribute Payload list"));
 
   silc_buffer_set(&buffer, (unsigned char *)payload, payload_len);
   list = silc_dlist_init();
 
-  while (buffer.len) {
+  while (silc_buffer_len(&buffer)) {
     newp = silc_calloc(1, sizeof(*newp));
     if (!newp)
       goto err;
     ret = silc_buffer_unformat(&buffer,
 			       SILC_STR_UI_CHAR(&newp->attribute),
 			       SILC_STR_UI_CHAR(&newp->flags),
-			       SILC_STR_UI16_NSTRING_ALLOC(&newp->data, 
+			       SILC_STR_UI16_NSTRING_ALLOC(&newp->data,
 							   &newp->data_len),
 			       SILC_STR_END);
     if (ret == -1)
       goto err;
 
-    if (newp->data_len > buffer.len) {
+    if (newp->data_len > silc_buffer_len(&buffer) - 4) {
       SILC_LOG_ERROR(("Incorrect attribute payload in list"));
       goto err;
     }
 
     len = 4 + newp->data_len;
-    if (buffer.len < len)
+    if (silc_buffer_len(&buffer) < len)
       break;
     silc_buffer_pull(&buffer, len);
 
     silc_dlist_add(list, newp);
   }
-  
+
   return list;
 
  err:
@@ -339,20 +374,21 @@ SilcBuffer silc_attribute_payload_encode_data(SilcBuffer attrs,
 					      SilcUInt32 data_len)
 {
   SilcBuffer buffer = attrs;
-  int len;
+  SilcUInt32 len;
 
-  len = 4 + data_len;
+  len = 4 + (SilcUInt16)data_len;
   buffer = silc_buffer_realloc(buffer,
-			       (buffer ? buffer->truelen + len : len));
+			       (buffer ? silc_buffer_truelen(buffer) +
+				len : len));
   if (!buffer)
     return NULL;
-  silc_buffer_pull(buffer, buffer->len);
+  silc_buffer_pull(buffer, silc_buffer_len(buffer));
   silc_buffer_pull_tail(buffer, len);
-  silc_buffer_format(buffer, 
+  silc_buffer_format(buffer,
 		     SILC_STR_UI_CHAR(attribute),
 		     SILC_STR_UI_CHAR(flags),
 		     SILC_STR_UI_SHORT((SilcUInt16)data_len),
-		     SILC_STR_UI_XNSTRING(data, data_len),
+		     SILC_STR_UI_XNSTRING(data, (SilcUInt16)data_len),
 		     SILC_STR_END);
   silc_buffer_push(buffer, buffer->data - buffer->head);
 
@@ -409,7 +445,7 @@ const unsigned char *silc_attribute_get_data(SilcAttributePayload payload,
 /* Construct digital signature verification data */
 
 unsigned char *silc_attribute_get_verify_data(SilcDList attrs,
-					      bool server_verification,
+					      SilcBool server_verification,
 					      SilcUInt32 *data_len)
 {
   SilcAttributePayload attr;
@@ -438,7 +474,7 @@ unsigned char *silc_attribute_get_verify_data(SilcDList attrs,
       if (!data)
 	return NULL;
       silc_buffer_set(&buffer, data + len, 4 + attr->data_len);
-      silc_buffer_format(&buffer, 
+      silc_buffer_format(&buffer,
 			 SILC_STR_UI_CHAR(attr->attribute),
 			 SILC_STR_UI_CHAR(attr->flags),
 			 SILC_STR_UI_SHORT(attr->data_len),
@@ -457,11 +493,11 @@ unsigned char *silc_attribute_get_verify_data(SilcDList attrs,
 
 /* Return parsed attribute object */
 
-bool silc_attribute_get_object(SilcAttributePayload payload,
-			       void *object, SilcUInt32 object_size)
+SilcBool silc_attribute_get_object(SilcAttributePayload payload,
+				   void *object, SilcUInt32 object_size)
 {
   SilcUInt16 len;
-  bool ret = FALSE;
+  SilcBool ret = FALSE;
 
   if (!object || payload->flags & SILC_ATTRIBUTE_FLAG_INVALID)
     return FALSE;
@@ -481,18 +517,32 @@ bool silc_attribute_get_object(SilcAttributePayload payload,
   case SILC_ATTRIBUTE_SERVICE:
     {
       SilcAttributeObjService *service = object;
+      SilcBufferStruct buf;
+      SilcUInt16 addr_len, signon_len;
+      char *addr, *signon;
+      int res;
       if (object_size != sizeof(*service))
 	break;
-      if (payload->data_len < 7)
+      if (payload->data_len < 13)
 	break;
-      SILC_GET32_MSB(service->port, payload->data);
-      SILC_GET16_MSB(len, payload->data + 4);
-      if (payload->data_len < 7 + len)
+      silc_buffer_set(&buf, payload->data, payload->data_len);
+      res = silc_buffer_unformat(&buf,
+				 SILC_STR_UI_INT(&service->port),
+				 SILC_STR_UI16_NSTRING(&addr, &addr_len),
+				 SILC_STR_UI_CHAR(&service->status),
+				 SILC_STR_UI16_NSTRING(&signon, &signon_len),
+				 SILC_STR_UI_INT(&service->idle),
+				 SILC_STR_END);
+      if (res == -1)
 	break;
-      memcpy(service->address, payload->data + 6,
-	     (len < sizeof(service->address) - 1 ? len :
+      memset(service->address, 0, sizeof(service->address));
+      memset(service->signon, 0, sizeof(service->signon));
+      memcpy(service->address, addr,
+	     (addr_len < sizeof(service->address) - 1 ? addr_len :
 	      sizeof(service->address) - 1));
-      service->status = payload->data[6 + len] ? TRUE : FALSE;
+      memcpy(service->signon, signon,
+	     (signon_len < sizeof(service->signon) - 1 ? signon_len :
+	      sizeof(service->signon) - 1));
       ret = TRUE;
     }
     break;
@@ -529,12 +579,13 @@ bool silc_attribute_get_object(SilcAttributePayload payload,
 
   case SILC_ATTRIBUTE_STATUS_MESSAGE:
   case SILC_ATTRIBUTE_EXTENSION:
+  case SILC_ATTRIBUTE_USER_ICON:
     {
-      SilcAttributeObjMime *mime = object;
+      SilcMime mime = object;
       if (object_size != sizeof(*mime))
 	break;
-      mime->mime = (const unsigned char *)payload->data;
-      mime->mime_len = payload->data_len;
+      if (!silc_mime_decode(mime, payload->data, payload->data_len))
+	break;
       ret = TRUE;
     }
     break;
@@ -585,6 +636,28 @@ bool silc_attribute_get_object(SilcAttributePayload payload,
     }
     break;
 
+  case SILC_ATTRIBUTE_PHONE_NUMBER:
+    {
+      SilcAttributeObjPN *pn = object;
+      SilcBufferStruct buffer;
+      SilcUInt32 pn_format;
+      int res;
+      if (object_size != sizeof(*pn))
+	break;
+      silc_buffer_set(&buffer, (unsigned char *)payload->data,
+		      payload->data_len);
+      res =
+	silc_buffer_unformat(&buffer,
+			     SILC_STR_UI_INT(&pn_format),
+			     SILC_STR_UI16_STRING_ALLOC(&pn->number),
+			     SILC_STR_END);
+      if (res == -1)
+	break;
+      pn->format = pn_format;
+      ret = TRUE;
+    }
+    break;
+
   case SILC_ATTRIBUTE_USER_PUBLIC_KEY:
   case SILC_ATTRIBUTE_SERVER_PUBLIC_KEY:
     {
@@ -599,7 +672,7 @@ bool silc_attribute_get_object(SilcAttributePayload payload,
 	silc_buffer_unformat(&buffer,
 			     SILC_STR_UI16_NSTRING_ALLOC(&pk->type, &len),
 			     SILC_STR_END);
-      if (res == -1)
+      if (res == -1 || len > silc_buffer_len(&buffer) - 2)
 	break;
       pk->data = silc_memdup(payload->data + 2 + len,
 			     payload->data_len - 2 - len);
